import json
import re
import urllib

__author__ = 'bromix'

import os
import xml.etree.ElementTree as ET


class SimpleAddon(object):
    def __init__(self, addon_path):
        self._path = addon_path

        self._json = {'name': '',
                      'version': '',
                      'config': []}
        self._id = ''
        self._icon = ''
        self._fanart = ''

        self.load()
        pass

    def to_json(self):
        return self._json

    def get_id(self):
        return self._id

    def get_icon(self):
        return self._icon

    def get_fanart(self):
        return self._fanart

    def get_version(self):
        return self._json['version']

    def get_name(self):
        return self._json['name']

    def has_config(self):
        return len(self._json['config']) > 0

    def get_config(self):
        return self._json['config']

    def supports_uri(self, uri):
        for config in self._json['config']:
            pattern = config['pattern']
            if re.match(pattern, uri):
                return True
            pass
        return False

    def create_addon_call(self, uri):
        for config in self._json['config']:
            pattern = config['pattern']
            if re.match(pattern, uri):
                subs = config.get('substitute', '')
                if subs:
                    return re.sub(pattern, subs, uri)

                return 'plugin://%s/uri2addon/?uri=%s' % (self._id, urllib.quote(uri))
            pass
        return ''

    def load(self):
        icon_path = os.path.join(self._path, 'icon.png')
        if os.path.exists(icon_path):
            self._icon = icon_path
            pass

        fanart_path = os.path.join(self._path, 'fanart.jpg')
        if os.path.exists(fanart_path):
            self._fanart = fanart_path
            pass

        uri2addon_json_filename = os.path.join(self._path, 'resources', 'uri2addon.json')
        if os.path.exists(uri2addon_json_filename):
            with open(uri2addon_json_filename) as uri2addon_json_file:
                self._json['config'].extend(json.load(uri2addon_json_file))
                pass
            pass

        """
        regex_path = os.path.join(self._path, 'resources', 'uri2addon')
        if os.path.exists(regex_path):
            with open(regex_path, 'r') as uri2addon_file:
                self._regex.extend(uri2addon_file.read().splitlines())
                pass
            pass
        """

        addon_xml = os.path.join(self._path, 'addon.xml')
        xml = ET.parse(addon_xml)
        root = xml.getroot()
        self._id = root.get('id', '')
        self._json['name'] = root.get('name', '')
        self._json['version'] = root.get('version', '')
        pass

    def get_path(self):
        return self._path

    pass


class AddonManager(object):
    def __init__(self, addons_path,
                 addon_type_filter=['metadata', 'plugin', 'skin', 'repository', 'script', 'resource', 'service']):
        self._addons_path = addons_path
        self._addons = []
        self._addon_type_filter = addon_type_filter

        self._json_data = {}
        pass

    def find_addons_for_uri(self, uri):
        result = []
        for addon in self._addons:
            if addon.supports_uri(uri):
                result.append(addon)
            pass
        return result

    def get_addons(self):
        return self._addons

    def update(self):
        for addon_id in os.listdir(self._addons_path):
            addon_id_parts = addon_id.split('.')
            if len(addon_id) > 0 and addon_id_parts[0] in self._addon_type_filter:
                full_addon_path = os.path.join(self._addons_path, addon_id)
                addon_xml = os.path.join(full_addon_path, 'addon.xml')
                if os.path.exists(addon_xml):
                    addon = SimpleAddon(full_addon_path)
                    if addon.has_config():
                        self._addons.append(addon)
                        self._json_data[addon.get_id()] = addon.to_json()
                        pass
                    pass
                pass
            pass
        pass

    pass