__author__ = 'bromix'

import os

from resources.lib.kodion.items.uri_item import UriItem
from resources.lib import kodion
from resources.lib.kodion.exceptions import KodionException
from resources.lib.uri2addon.addon_manager import AddonManager


class Provider(kodion.AbstractProvider):
    def __init__(self):
        kodion.AbstractProvider.__init__(self)

        # update localization
        self._local_map.update(
            {
                'uri2addon.please_select_addon_for_execution': 30500,
                'uri2addon.no_matching_addon_for_uri_found': 30501
            }
        )
        pass

    @kodion.RegisterProviderPath('^/open/$')
    def on_open_url(self, context, re_match):
        uri = context.get_param('uri', '')
        if not uri:
            raise KodionException('Missing query parameter "uri" for /play/')

        addons_path = context.get_native_path()
        addons_path = os.path.dirname(addons_path)

        addon_manager = AddonManager(addons_path, ['plugin'])
        addon_manager.update()
        found_addons = addon_manager.find_addons_for_uri(uri)

        # no addon found for the URI
        if len(found_addons) == 0:
            context.get_ui().on_ok(title=context.localize(self._local_map['uri2addon.no_matching_addon_for_uri_found']),
                                   text=uri)
            context.log_error('Could not find addon for the given URI "%s"' % uri)
            pass

        selected_addon = None
        if len(found_addons) == 1:
            selected_addon = found_addons[0]
            pass
        elif len(found_addons) > 1:
            select_items = []
            for found_addon in found_addons:
                select_items.append((found_addon.get_name(), found_addon))
                pass

            # provide a selection of matched addons
            selected_addon = context.get_ui().on_select(
                context.localize(self._local_map['uri2addon.please_select_addon_for_execution']), select_items)
            if not selected_addon or selected_addon == -1:
                selected_addon = None
            pass

        # call the endpoint of the found/selected addon
        if selected_addon:
            new_uri = selected_addon.create_addon_call(uri)
            return UriItem(new_uri)

        return False

    def on_root(self, context, re_match):
        addons_path = context.get_native_path()
        addons_path = os.path.dirname(addons_path)

        addon_manager = AddonManager(addons_path, ['plugin'])
        addon_manager.update()
        addons = addon_manager.get_addons()
        for addon in addons:
            context.log_notice(u'Testing addon: "%s"' % addon.get_name())
            pass

        return True

    pass